@@ -50,6 +50,13 @@ class BulkOperations:
             for record in data:
                 record["tenant_id"] = tenant_id
 
+        # Auto-convert ISO datetime strings to datetime objects for each record
+        from ..core.nodes import convert_datetime_fields
+
+        model_fields = self.dataflow.get_model_fields(model_name)
+        for record in data:
+            convert_datetime_fields(record, model_fields, logger)
+
         # Perform actual database insertion
         try:
             connection_string = self.dataflow.config.database.get_connection_url(
@@ -215,6 +222,14 @@ class BulkOperations:
             # Filter-based bulk update - perform actual database operation
             logger.warning("BULK_UPDATE: Processing filter-based update")
             try:
+                # Auto-convert ISO datetime strings to datetime objects in update_values
+                from ..core.nodes import convert_datetime_fields
+
+                model_fields = self.dataflow.get_model_fields(model_name)
+                update_values = convert_datetime_fields(
+                    update_values, model_fields, logger
+                )
+
                 # Get database connection and execute UPDATE
                 connection_string = self.dataflow.config.database.get_connection_url(
                     self.dataflow.config.environment
@@ -320,6 +335,13 @@ class BulkOperations:
                     "success": True,
                 }
 
+            # Auto-convert ISO datetime strings to datetime objects for each record
+            from ..core.nodes import convert_datetime_fields
+
+            model_fields = self.dataflow.get_model_fields(model_name)
+            for record in data:
+                convert_datetime_fields(record, model_fields, logger)
+
             try:
                 connection_string = self.dataflow.config.database.get_connection_url(
                     self.dataflow.config.environment
@@ -591,6 +613,13 @@ class BulkOperations:
             for record in data:
                 record["tenant_id"] = tenant_id
 
+        # Auto-convert ISO datetime strings to datetime objects for each record
+        from ..core.nodes import convert_datetime_fields
+
+        model_fields = self.dataflow.get_model_fields(model_name)
+        for record in data:
+            convert_datetime_fields(record, model_fields, logger)
+
         # Simulate upsert with conflict resolution
         total_records = len(data)
         inserted = int(total_records * 0.7)  # 70% new records
